@@ -32,6 +32,19 @@
 
 #include "glusterfs-fops.h" /* generated XDR values for FOPs */
 
+#ifndef IXDR_GET_LONG
+#define IXDR_GET_LONG(buf) ((long)IXDR_GET_U_INT32(buf))
+#endif
+#ifndef IXDR_PUT_LONG
+#define IXDR_PUT_LONG(buf, v) ((long)IXDR_PUT_INT32(buf, (long)(v)))
+#endif
+#ifndef IXDR_GET_U_LONG
+#define IXDR_GET_U_LONG(buf) ((u_long)IXDR_GET_LONG(buf))
+#endif
+#ifndef IXDR_PUT_U_LONG
+#define IXDR_PUT_U_LONG(buf, v) IXDR_PUT_LONG(buf, (long)(v))
+#endif
+
 #include "list.h"
 #include "locking.h"
 #include "logging.h"
@@ -72,6 +85,9 @@
 #define FNM_EXTMATCH 0
 #endif
 
+/*gets max-offset on all architectures correctly*/
+#define GF_OFF_MAX ((1ULL << (sizeof(off_t) * 8 - 1)) - 1ULL)
+
 #define GLUSTERD_MAX_SNAP_NAME  255
 #define GLUSTERFS_SOCKET_LISTEN_BACKLOG  10
 #define ZR_MOUNTPOINT_OPT       "mountpoint"
@@ -272,6 +288,7 @@
 #define TIER_LINKFILE_GFID           "tier-linkfile-gfid"
 #define DHT_SKIP_OPEN_FD_UNLINK     "dont-unlink-for-open-fd"
 #define DHT_IATT_IN_XDATA_KEY       "dht-get-iatt-in-xattr"
+#define DHT_MODE_IN_XDATA_KEY       "dht-get-mode-in-xattr"
 #define GET_LINK_COUNT              "get-link-count"
 #define GF_GET_SIZE                 "get-size"
 
@@ -297,8 +314,9 @@
 #define GF_BACKTRACE_LEN        4096
 #define GF_BACKTRACE_FRAME_COUNT 7
 
-#define GF_LK_ADVISORY 0
-#define GF_LK_MANDATORY 1
+#define GF_LK_ADVISORY 0 /* maps to GLFS_LK_ADVISORY from libgfapi*/
+#define GF_LK_MANDATORY 1 /* maps to GLFS_LK_MANDATORY from libgfapi*/
+#define GF_LOCK_MODE "glusterfs.lk.lkmode"
 
 const char *fop_enum_to_pri_string (glusterfs_fop_t fop);
 
@@ -306,6 +324,7 @@ const char *fop_enum_to_pri_string (glusterfs_fop_t fop);
 #define GF_SET_OVERWRITE      0x2 /* Overwrite with the buf given */
 #define GF_SET_DIR_ONLY       0x4
 #define GF_SET_EPOCH_TIME     0x8 /* used by afr dir lookup selfheal */
+#define GF_AUXILLARY_PARGFID  0xd /* RIO dummy parent gfid */
 
 /* key value which quick read uses to get small files in lookup cbk */
 #define GF_CONTENT_KEY "glusterfs.content"
@@ -422,7 +441,9 @@ struct _cmd_args {
         /* For the subdir mount */
         char           *subdir_mount;
 
+        char              *process_name;
         char              *event_history;
+        int                thin_client;
 };
 typedef struct _cmd_args cmd_args_t;
 
@@ -502,11 +523,9 @@ struct _glusterfs_ctx {
         int                process_mode; /*mode in which process is runninng*/
         struct syncenv    *env;          /* The env pointer to the synctasks */
 
-#if defined(OLD_MEM_POOLS)
         struct list_head   mempool_list; /* used to keep a global list of
                                             mempools, used to log details of
                                             mempool in statedump */
-#endif
         char              *statedump_path;
 
         struct mem_pool   *dict_pool;
@@ -552,6 +571,11 @@ struct _glusterfs_ctx {
 
         gf_lock_t          volfile_lock;
 
+        /* configuration related elements, which gets changed
+           from global xlator */
+        struct {
+                char *metrics_dumppath;
+        } config;
 
         struct {
                 gf_atomic_t max_dict_pairs;
